
// (c) COPYRIGHT URI/MIT 1999
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation for IDLStr. See IDLByte.cc
//
// jhrg 9/25/96

// $Log: IDLStr.cc,v $
// Revision 1.1.2.1  1999/09/23 23:06:31  jimg
// This collection of files builds under Solaris 2.6. The resulting idl_dods.so
// runs correctly on the fnoc1.nc test dataset.
//

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_dap.h"

#include <assert.h>
#include <string.h>
#if 0
#include <math.h>
#include <stdlib.h>

#include "config_writeval.h"		// was first caused LITTLE_ENDIAN error
#endif

#if 0
#include <iostream>
#endif
#include <string>

#if 0
#include <Pix.h>
#include <SLList.h>
#include <Regex.h>
#endif

#include "IDLStr.h"
#if 0
#include "name_map.h"
#endif

#if 0
extern "C" {
#include "rx.h"
}
#endif

// We *should* be using <nan.h> to handle generating NaN. However, many
// machines don't have a function that can be used to generate NaN so I've
// written my own (which uses its own bastardized version of the FP union).
// 2/5/97 jhrg

#if 0
#include "nan_hack.h"

static double
MakeNaN() 
{ 
    dnan X;
    X.nan.exponent = 0x7ff; 
    X.nan.qnan_bit = 0x1; 
    
    return X.d;
}

extern name_map names;
extern bool translate;
extern bool ascii;
extern bool string_to_float;
extern bool dms_to_float;
extern bool verbose;
extern bool warning;
#endif

Str *
NewStr(const string &n)
{
    return new IDLStr(n);
}

IDLStr::IDLStr(const string &n) : Str(n)
{
}

BaseType *
IDLStr::ptr_duplicate()
{
    return new IDLStr(*this);
}

bool
IDLStr::read(const string &, int &)
{
    assert(false);
    return false;
}

// Is the string a latitude or longitude in degrees-minutes-seconds form?
// Check for a string of the form DDDMMSSH where DDDMMSS are all digits 0-9
// and H is one of N,S,E,W or the lower case equivalents. If #value# fits
// those criteria, return true, otherwise return false.

#if 0
static bool
really_dms(string value)
{
    Regex r("[0-9][0-9]?[0-9]?[0-5][0-9][0-5][0-9][nsewNSEW]");
    
    return r.match(value.c_str(), value.length()) > 0;
#if 0
	int rc, len;
	regex_t dms;
	regmatch_t pm;		// need this, even though we shouldn't
	char target[20];

	len = value.size();		// make it C 
	value.copy(target, len);
	target[len] = 0;
	// compile the pattern
    	regcomp(&dms, "[0-9][0-9]?[0-9]?[0-5][0-9][0-5][0-9][nsewNSEW]", 
		REG_EXTENDED);
	
	rc = regexec(&dms, &target[0], 1, &pm, 0);	// does it match
	regfree(&dms);			// free up the pattern
	if (rc == 0)
		return true;
	return false;
#endif
}

// Given that the string #value# is of the form DDDMMSSH, convert it to
// fractional degrees representation (where MMSS is converted into a fraction
// of a degree). Set the sign so that Latitude North is positive, Latitude
// South is negitive, Longitude East is positive and Longitude West is
// negative.
//
// NB: Assume that the MMSSH part is constant length (i.e., 0 min 0 sec is
// represented 0000) and that N,S,W,E, ... is always present.

static double
convert_dms(string value)
{
    int len = value.size();	// Handle variable length in the degree part.
    string hemisphere = value.substr(len-1, 1);
    string seconds = value.substr(len-3, 2);
    string minutes = value.substr(len-5, 2);
    string degrees = value.substr(0, len-5);

#if DODS_DEBUG
    cerr << "Hemi: " << hemisphere.data() << endl
	<< "Sec: " << seconds.data() << endl
	<< "Min: " << minutes.data() << endl
	<< "Deg: " << degrees.data() << endl;
#endif

    double secs = atoi(seconds.data()) / 60.0;
    double mins = (atoi(minutes.data()) + secs)/ 60.0;
    double val = (double)atoi(degrees.data()) + mins;

    if (hemisphere.data() == "s" || hemisphere.data() == "w"
    	|| hemisphere.data() == "S" || hemisphere.data() == "W")
	val *= -1.0;

#if DODS_DEBUG
    cerr << "Val: " << val << " (mins : " << mins << " secs: " << secs
	 << ")" <<endl;
#endif

    return val;
}

void 
IDLStr::print_val(ostream &os, string, bool print_decl_p)
{
    bool print_as_float = false;
    double val = 0.0;

    // Old degree-min-sec translation code, an anachronism. jhrg 1/9/98.
    if (dms_to_float && really_dms(_buf)) {
	val = convert_dms(_buf);
	print_as_float = true;
    }

    // Translate all string variables to Floats. jhrg 1/9/98.
    if (string_to_float) {
	char *ptr = NULL;
	const char *str = _buf.c_str();
	val = strtod(str, &ptr);
	print_as_float = !(val == 0.0 && (ptr == str));

	if (!print_as_float) {
	    val = MakeNaN();
	    print_as_float = true;
	    if (warning) {
		cerr << "Could not translate `" << _buf << "' to a Float64,"
		     << endl;
		cerr << "interning as NaN (not a number: " << val 
		     << ")" << endl;
	    }
	}
    }

    if (print_as_float) {
	if (print_decl_p)
	    os << "Float64" << endl << names.lookup(name(), translate) << endl;

	if (ascii)
	    os << val << " ";
	else
	    os.write((void *)&val, sizeof(double));
    }
    else {
	if (print_decl_p)
	    os << type_name() << endl << names.lookup(name(), translate) 
		<< endl;

	os << _buf << endl;
    }
}
#endif
